from pettingzoo.utils.depreciated_module import DepreciatedModule

multiwalker_v0 = DepreciatedModule("multiwalker", "v0", "v2")
multiwalker_v0 = DepreciatedModule("multiwalker", "v0", "v2")
pursuit_v0 = DepreciatedModule("pursuit", "v0", "v1")
waterworld_v0 = DepreciatedModule("waterworld", "v0", "v1")
